import { EntityNodeAttributes, RelationNodeAttributes } from "../model/graphology/model";
import { QueryMultiGraph } from "../model/graphology/utils";
import { BackendQueryFormat } from "../model/BackendQueryFormat";
import { Handles, QueryElementTypes } from "../model";

/**
   * Converts the ReactFlow query to a json data structure to send the query to the backend.
   * @returns {BackendQueryFormat} A JSON object in the `JSONFormat`.
   */
export function Query2BackendQuery(databaseName: string, graph: QueryMultiGraph): BackendQueryFormat {
    // dict of nodes per type
    const entities = graph.nodes.filter(n => n.attributes?.type === QueryElementTypes.Entity)
        .map((n) => ({ name: n.attributes!.name, ID: Number(n.key), constraints: [] }));

    const relations = graph.nodes.filter(n => n.attributes?.type === QueryElementTypes.Relation)
        .map((n) => {
            const attributes = n.attributes as RelationNodeAttributes;
            const leftEdge = graph.edges.filter(e => (e.target === n.key && e.attributes!.targetHandle === Handles.RelationLeft))?.[0];
            const rightEdge = graph.edges.filter(e => (e.target === n.key && e.attributes!.targetHandle === Handles.RelationRight))?.[0];
            if (!leftEdge || !rightEdge) throw Error('Malformed Graph! One or more edges of a relation node do not exist');

            const leftType = graph.nodes.filter(n => n.key === leftEdge.source)?.[0]?.attributes?.type;
            const rightType = graph.nodes.filter(n => n.key === rightEdge.source)?.[0]?.attributes?.type;
            if (!rightType || !leftType) throw Error('Malformed Graph! edges must have a type');

            return {
                name: attributes!.collection, ID: Number(n.key), depth: attributes!.depth,
                fromType: leftType, fromID: Number(leftEdge.source), // need to treat function==groupby
                toType: rightType, toID: Number(rightEdge.source), // need to treat function==groupby
                constraints: [],
            }
        });

    // TODO: Groubby not implemented (constraints)
    const result: BackendQueryFormat = {
        databaseName: databaseName,
        return: { entities: entities.map(e => e.ID), relations: relations.map(r => r.ID), groupBys: [] },
        entities: entities,
        relations: relations,
        groupBys: [], // TODO
        machineLearning: [], // TODO
        limit: 500, // TODO
    };
    console.log(result, graph);


    //note that attribute nodes are not processed here, but are detected if they are connected to any entity/relation/function node

    //add nodes to JSON query
    // entityNodes.forEach((node) => {
    //     this.AddEntityToJsonObject(result, node);
    // });
    // relationNodes.forEach((node) => {
    //     this.AddRelationToJsonObject(result, node);
    // });

    // functionNodes.forEach((functionNode: FunctionNode) => {
    //   switch (functionNode.data?.functionType) {
    //     case FunctionTypes.GroupBy:
    //       this.AddGroupByToJsonObject(result, functionNode);
    //       break;
    //     case FunctionTypes.link:
    //       this.AddLinkPredictionToJsonObject(result, functionNode);
    //       break;
    //     case FunctionTypes.communityDetection:
    //       this.AddCommunityDetectionToJsonObject(result, functionNode);
    //       break;
    //     case FunctionTypes.centrality:
    //       this.AddCentralityToJsonObject(result, functionNode);
    //       break;
    //     case FunctionTypes.shortestPath:
    //       this.addShortestPathToJsonOBject(result, functionNode);
    //       break;
    //   }
    // });
    return result;
}